/*
 * gc global variable provides access to GUI Composer infrastructure components and project information.
 * For more information, please see the Working with Javascript guide in the online help.
 */
 var gc = gc || {};
 
/*
 * wait for gc.databind to be ready. 
 */
document.addEventListener('gc-databind-ready', function() {

    /* 
     * To see the console logs logged by this file, in the console view, enter the following:
     * gc.console.setCookie('CustomCodec',5) 
     * For more help on how to work with gc.console, please see the help text that is sent to the
     * console when the app starts.
     */
    
    /*
     * Boilerplate code for encoding and decoding packet data transmitted to / received from TI devices.
     * For an example of a codec that accesses symbols using readValue/writeValue, please see components/ti-widget-registers/demo/MotorwareDemo_F28069M/customCodec_0.js
     */
    
    /**
     * This code implements the IPacketCodec interface.
     *
     * @constructor
     * @implements gc.databind.IPacketCodec
     */
    CustomCodec = function() {};
    CustomCodec.prototype = new gc.databind.IPollingPacketCodec();
    CustomCodec.prototype.isConnected = false;
    CustomCodec.prototype.initComplete = false;
    CustomCodec.prototype.templateObj = undefined;

    CustomCodec.prototype.partialData = [];
    CustomCodec.prototype.checkData = 0;
    
    /**
     * Encodes data into a packet for sending to the target device.
     *
     * @param target - function to call to pass on the encoded data towards the target
     * @param jsonObj - The data object from the GUI that is to be encoded into a packet for sending.
     */
    CustomCodec.prototype.encode = function(target, somedata) {
        target(somedata);
    };

    const lookFor = [0xc1, 0xd4, 0x0, 0x0];
    function findStart(data) {
        let lookAt=0, idx;
        for (idx=0; idx<data.length; idx++) {
            if (data[idx] === lookFor[lookAt]) {
                lookAt++;
                if (lookAt === lookFor.length) {
                    return (idx+1-lookAt);
                }
            } else {
                lookAt = 0;
            }
        }
        if (lookAt > 0) {
            return idx-lookAt;
        } else {
            return -1;
        }
    }

    CustomCodec.prototype.process = function(data) {
        let posHint = 0;
        while (posHint < data.length) {
            try {
                this.global_i++;
                //console.log(this.global_i, posHint, data[posHint]);
                let decodedResult = msgpack.deserialize(data, posHint);
                //console.log(this.global_i, decodedResult.nextIndex);
                posHint = decodedResult.nextIndex;
                const decodedData = decodedResult.data;

                if (decodedData instanceof Array) {
                    this.templateObj.$.ti_widget_value1.value=decodedData[0];
                    this.templateObj.$.ti_widget_value2.value=decodedData[1];
                    this.templateObj.$.ti_widget_value3.value=decodedData[2];
                } else {
                    // assume it is an object of 3 key-value pairs
                    dummy=0;
                    for (let aKey in decodedData) {
                        if (dummy===0) {
                            this.templateObj.$.ti_widget_key1.value=aKey;
                            this.templateObj.$.ti_widget_value1.value=decodedData[aKey];
                        } else if (dummy===1) {
                            this.templateObj.$.ti_widget_key2.value=aKey;
                            this.templateObj.$.ti_widget_value2.value=decodedData[aKey];
                        } else if (dummy===2) {
                            this.templateObj.$.ti_widget_key3.value=aKey;
                            this.templateObj.$.ti_widget_value3.value=decodedData[aKey];
                        }
                        dummy++;
                    }
                }
                
            } catch (error) {
                //console.log(this.global_i, error, posHint, data[posHint]);
                break;
            }
        }
        return posHint;
    }

    /**
     * Decodes packets from the target device into data objects.  One object for each packet of data.
     * Unless you have chained your custom codec with the CR codec, the packet data is not framed,
     * so this method is responsible for decoding partial packets as well as multiple packets.  Partial packets
     * should not be returned, but deferred until the next decode method call with more raw data to process.  As a
     * result, this method may return 0 or more packets of data on each invocation.
     *
     * @param target -  function to call to pass on a JSON object containing the decoded values towards the GUI
     * @param data  - The raw data received that needs to be decoded from packets into objects.
     * @return - true if connected (i.e. valid data received), false if not connected
     */
    CustomCodec.prototype.decode = function(target, data) {
        this.isConnected = true;
        if (this.checkData <= 0) return this.isConnected;

        this.partialData.push(...data);
        //this.gloabl_array.push(...data);
        if (this.checkData === 1) {
            const foundStart = findStart(this.partialData);
            if (foundStart < 0) {
                this.partialData = [];
            } else if (this.partialData.length < foundStart + lookFor.length) {
                this.partialData = this.partialData.slice(foundStart);
            } else {
                this.partialData = this.partialData.slice(foundStart + lookFor.length);
                this.checkData = 2;
            }
        }
        if (this.checkData === 2) {
            const next = this.process(this.partialData);
            if (next > 0) {
                this.partialData = this.partialData.slice(next);
            }
       }
       // Return true to set the 'Hardware Connected' status in the status bar
       return this.isConnected;
    };
    
    CustomCodec.prototype.disconnect = function(){
        this.initComplete = false;
        this.isConnected = false;
        this.partialData=[];
        this.checkData=0;
        // TODO: remove any event listeners you added in the connect function here:
    };
    
    CustomCodec.prototype.connect = function(settings) {
        this.templateObj = document.querySelector('#template_obj');
        if (this.templateObj) {
             // You can use 'automatic node finding' $ syntax with the templateObj to access widgets in your app.
            // e.g. to access a widget with an id of 'widget_id' you can use templateObj.$.widgetId
            // TODO: Add any event listeners or initialization logic this codec requires here:
        }
        else {
            gc.console.log("CustomCodec","CustomCodec: Failed to find id template_obj in the Application!");
        }

        window.mycodec = this; // for chrome browser console to debug mycodec functions

        this.initComplete = true;
        this.partialData=[];
        this.checkData = 1;
        this.encoder('s'); // to start hw to send host some data; 'e' is to stop hw to send host data;
        this.global_i=0; // for debug only
        this.gloabl_array=[]; // for debug only

        return Q(); // return a promise
    };

    CustomCodec.prototype.resync = function() {
        this.checkData=0;
        this.encoder('e');
        this.partialData=[];
        this.checkData=1;
        this.encoder('s');
    };
    
    CustomCodec.prototype.sendSomeData = function() {
        //const message = msgpack.encode(40); // encode an integer
        
        //const message = msgpack.encode(['abc', 2, 1.23]); // encode an array
        
        const message = msgpack.encode({foo: 1, bar: 'abc', tmp: 1.234}); // encode key value pairs
        // message is UInt8Array. Don't know why the HW side using UART_read(uart, inputBuffer, 255) does not get data.
        // convert it to js number array and UART_read can see the data
        this.encoder(Array.from(message));
    }

    
    // registerCustomCodec has to be called on or before then gc-databind-ready event in order for the 
    // codec to be recognized and used
    gc.console.log("CustomCodec","CustomCodec","CustomCodec: about to register...");
    // You can 'chain' codecs together to make it easier to work with the data
    // coming from the device or going to the device.
    // The following codecs are provided:
    //   CR  - use this if the target data uses /n as a delimiter for framing.
    //         The CR codec will take care of buffering the incoming data so that
    //         your codec will only receive complete 'ready to use' packets
    //  If your target uses /n delimited strings, set baseCodecs = "CR"
    //  If your target uses some other format for data (e.g. binary data or custom strings) leave baseCodecs = "undefined"
    var baseCodecs;
    gc.databind.registerCustomCodec("custom",CustomCodec, baseCodecs);

});


var onResync = function() {

    if (gc && gc.databind) {
        const c = gc.databind.registry.getModel('ti_model_streaming').getCodec();
        c && c.resync();
    }

}

var onSendSomeData = function() {
    if (gc && gc.databind) {
        const c = gc.databind.registry.getModel('ti_model_streaming').getCodec();
        c && c.sendSomeData();
    }
    
}

